<?php

define('DEFAULT_BIBLE_TRANSLATION','NIV');

function scripturize ($text = '',$bible = DEFAULT_BIBLE_TRANSLATION){
    // skip everything within a hyperlink, a <pre> block, a <code> block, or a tag
    // we skip tags because something like <img src="blah" alt="John 3:16"> should not be messed with
    $anchor_regex = '<a\s+href.*?<\/a>';
    $pre_regex = '<pre>.*<\/pre>';
    $code_regex = '<code>.*<\/code>';
    $tag_regex = '<(?:[^<>\s]*)(?:\s[^<>]*){0,1}>'; // $tag_regex='<[^>]+>';
    $split_regex = "/((?:$anchor_regex)|(?:$pre_regex)|(?:$code_regex)|(?:$tag_regex))/i";

    $parsed_text = preg_split($split_regex,$text,-1,PREG_SPLIT_DELIM_CAPTURE);
    $linked_text = '';

    while (list($key,$value) = each($parsed_text)) {
        if (preg_match($split_regex,$value)) {
	    $linked_text .= $value; // if it is an HTML element or within a link, just leave it as is
	} else {
            $linked_text .= scripturizeAddLinks($value,$bible); // if it's text, parse it for Bible references
	}
    }
    
    return $linked_text;
}

function scripturizeAddLinks($text = '',$bible = DEFAULT_BIBLE_TRANSLATION) {
    $volume_regex = '1|2|3|I|II|III|1st|2nd|3rd|First|Second|Third';

    $book_regex  = 'Genesis|Exodus|Leviticus|Numbers|Deuteronomy';
    $book_regex .= '|Joshua|Judges|Ruth|Ezra|Nehemiah|Esther';
    $book_regex .= '|Job|Psalms?|Proverbs?|Ecclesiastes';
    $book_regex .= '|Songs? of Solomon|Song of Songs';
    $book_regex .= '|Isaiah|Jeremiah|Lamentations|Ezekiel|Daniel';
    $book_regex .= '|Hosea|Joel|Amos|Obadiah|Jonah|Micah|Nahum';
    $book_regex .= '|Habakkuk|Zephaniah|Haggai|Zechariah|Malachi';
    $book_regex .= '|Mat+hew|Mark|Luke|John|Acts?|Acts of the Apostles';
    $book_regex .= '|Romans|Galatians|Ephesians|Phil+ippians|Colossians';
    $book_regex .= '|Titus|Philemon|Hebrews|James|Jude|Revelations?';

    $volume_book_regex  = 'Samuel|Kings|Chronicles|Corinthians';
    $volume_book_regex .= '|Thessalonians|Timothy|Peter|John';

    // split these up from the Perl code because I want to be able to have an optional period at the end of just the abbreviations

    $abbrev_regex  = 'Gen|Ex|Exo|Lev|Num|Nmb|Deut?';
    $abbrev_regex .= '|Josh?|Judg?|Jdg|Rut|Ezr|Neh';
    $abbrev_regex .= '|Est|Jb|Psa?|Pr(?:ov?)?';
    $abbrev_regex .= '|Eccl?|Song?(?: of Sol)?';
    $abbrev_regex .= '|Isa|Jer|Lam|Eze|Dan';
    $abbrev_regex .= '|Hos|Joe|Amo|Oba|Jon|Mic|Nah';
    $abbrev_regex .= '|Hab|Zeph?|Hag|Zech?|Mal';
    $abbrev_regex .= '|Mat?t|Mr?k|Lu?k|Jh?n|Jo|Act';
    $abbrev_regex .= '|Rom|Gal|Eph|Phi|Phil|Col|Tim';
    $abbrev_regex .= '|Tit|Phile|Heb|Ja?m|Ju?d|Rev';

    $volume_abbrev_regex = 'Sam|Ki?n|Chr(?:on?)?|Jo|Jh?n';
    $volume_abbrev_regex .= '|Cor|The?|Thess?|Tim|Pe?t';

    $book_regex = '(?:' . $book_regex . ')|(?:' . $abbrev_regex . ')\.?';
    $book_regex_withvolume = '(?:'. $volume_book_regex . ')|(?:' . $volume_abbrev_regex . ')\.?';

    $verse_regex="\d{1,3}(?::\d{1,3})?(?:\s?(?:[-&,]\s?(?:\d{1,3}:)?\d+))*";

    $translation_regex = 'NIV|NIV1984|NASB|AMP|NLT|KJV|ESV|CEV|NET|NKJV|KJ21|ASV|WE|YLT|DARBY|WYC|NIV-UK|TNIV|MSG|NIRV';

    // note that this will be executed as PHP code after substitution thanks to the /e at the end!

    $passage_regex = '/(?:(?:(' . $book_regex . '))|(?:(' . $volume_regex . ')\s)(' . $book_regex_withvolume . '))\s';
    $passage_regex .= '(' . $verse_regex . ')(?:\s?[,-]?\s?((?:' . $translation_regex . ')|\s?\((?:' . $translation_regex . ')\)))?/e';
    $replacement_regex = "scripturize_link_reference('\\0','\\1','\\2','\\3','\\4','\\5','$bible')";

    $text=preg_replace($passage_regex,$replacement_regex,$text);

    return $text;
}

function scripturize_link_reference($reference='', $book1='', $volume='', $book2='', $verse='', $translation='', $user_translation='') {
  $book = ($book1=='')?$book2:$book1;
    if ($volume) {
       $volume = str_replace('III','3',$volume);
       $volume = str_replace('II','2',$volume);
       $volume = str_replace('I','1',$volume);
       $volume = $volume{0}; // will remove st,nd,and rd (presupposes regex is correct)
    }

   if(!$translation) {
         if (!$user_translation) {
             $translation = DEFAULT_BIBLE_TRANSLATION;
         } else {
             $translation = $user_translation;
         }
   } else {
       $translation = trim($translation,' ()'); // strip out any parentheses that might have made it this far
   }


   // Sometimes biblegateway.com changes the abbreviation for a translation. This canonicalises them without breaking sites using older abbreviations.
   $trans_replace = array(
     'NIV-UK' => 'NIVUK');
   if (isset($trans_replace[$translation])) $translation = $trans_replace[$translation];

   // if necessary, just choose part of the verse reference to pass to the web interfaces
   // they wouldn't know what to do with John 5:1-2, 5, 10-13 so I just give them John 5:1-2
   // this doesn't work quite right with something like 1:5,6 - it gets chopped to 1:5 instead of converted to 1:5-6
   if ($verse) {
       $verse = strtok($verse,',& ');
   }

   $book = trim($book, '.');

   // Some descriptors for Bible books may be unknown to the websites providing the text.
   // Replace these if necessary. To add such replacements, add new entries to this array
   $book_replace = array(
    'Phi' => 'Phil',
    'Song of Sol' => 'Song of Songs');
   if (isset($book_replace[$book])) $book = $book_replace[$book];

   switch ($translation) {
        case 'ESV':
        // note: the ESV could actually support a mouseover reference
        // we could pull it directly from their site and include it in the $title text
        // http://www.gnpcb.org/esv/share/services/api/ for more info
             $link = htmlspecialchars('http://www.gnpcb.org/esv/search/?go=Go&q=');
             $title = 'English Standard Version Bible';
             $link = sprintf('<a href="%s%s" title="%s">%s</a>',$link,htmlentities(urlencode(trim("$volume $book $verse"))),$title,trim($reference));
             break;
        case 'NET':
             $link = htmlspecialchars('http://net.bible.org/passage.php?passage=');
             $title = 'New English Translation';
             $link = sprintf('<a href="%s%s" title="%s">%s</a>',$link,htmlentities(urlencode(trim("$volume $book $verse"))),$title,trim($reference));
             break;
        default:
             $link = "http://biblegateway.com/passage/?search=";
             $translation_suffix = htmlspecialchars('&version=');
             $title = 'Bible Gateway';
             $link = sprintf('<a href="%s%s%s%s" title="%s">%s</a>', $link, htmlentities(urlencode(trim("$volume $book $verse"))), $translation_suffix, $translation, $title, trim($reference));
             break;
    }

    return $link;
}

?>
